//Chameleon_UrJTAG_CPLD.C Doug Commons 2008
//
//This design has been ported from the usb_jtag design presented by 
//Kolja Waschk Copyright (C) 2005-2007 Kolja Waschk, ixo.de
//The code was re-written for the AVR processor "ATMega32U4" and has a number 
//of changes that are not present in the original design. The introductory 
//comments were taken from Kolja's design. This code has 
//currently been tested with the XC9536XL, XC9572XL CPLDs and the XC3S50AN
//FPGA and works well.
//-----------------------------------------------------------------------------
//
//Chameleon_UrJTAG_CPLD is free software; you can redistribute it and/or modify 
//it under the terms of the GNU General Public License as published by the Free 
//Software Foundation; either version 2 of the License, or (at your option) any 
//later version.
//
//Chameleon_UrJTAG_CPLD is distributed in the hope that it will be useful, but 
//WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
//FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
//details.
//
//You should have received a copy of the GNU General Public License
//along with this program as the file COPYING.txt; if not, write to the Free 
//Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 
//USA
//-------------------------------------------------------------------------
// usb_jtag_activity does most of the work. It behaves just 
// like the combination of FT245BM and Altera-programmed EPM7064 CPLD in 
// Altera's USB-Blaster. The CPLD knows two major modes: Bit banging mode 
// and unsigned char shift mode. It starts in Bit banging mode. While bytes 
// are received from the host on EP2 OUT, each unsigned char cmd is 
// processed as follows:
//
// Bit banging mode:
// 	 0. Read command byte and set the clockbytes so they equal the LSB 6 
//		bits. This is the number of subsequent shift transfers to execute.
//
//   1. Remember bit 6 (0x40) in B as the "Read bit".
//
//   2. If bit 7 (0x40) is set, switch to Shift mode for the
//      next X bytes ( X := B & 0x3F ), and don't do anything else now.
//
//   3. Otherwise, set the JTAG signals as follows:
//      TCK high if bit 0 was set (0x01), otherwise low
//      TMS high if bit 1 was set (0x02), otherwise low
//      nCE bit 2 is unsupported
//      nCS bit 3 is unsupported
//      TDI high if bit 4 was set (0x10), otherwise low
//      LED high if bit 5 was set (0x20), otherwise low
//
//   4. If "Read bit" (0x40) was set, record the state of TDO
//		pin and put it as a unsigned char 
//		TDO in the output BufferOut FIFO "to" the host 
//
// Shift mode:
//
//   1. Load shift register with unsigned char "DATAOUT" from host
//
//   2. Do 8 times (i.e. for each bit of the unsigned char)
//      2a) if nCS=1, set carry bit from TDO, else set carry bit from
//			DATAOUT
//      2b) Rotate shift register through carry bit
//      2c) TDI = Carry bit
//      2d) Raise TCK, then lower TCK.
//
//   3. If "Read bit" was set when switching into unsigned char shift mode,
//      record the shift register content and put it into the BufferOUT FIFO
//      "to" the host.
//
// Some more (minor) things to consider to emulate the FT245BM:
//
//   a) The FT245BM seems to transmit just packets of no more than 64 
//		bytes (which perfectly matches the USB spec). Each packet starts 
//		with two non-data bytes (I use 0x31,0x60 here). A USB sniffer on 
//		Windows might show a number of packets to you as if it was a large 
//		transfer because of the way that Windows understands it: it's a
//		large transfer until terminated with an USB packet smaller than 64 
//		unsigned char.
//
//   b) The Windows driver expects to get some data packets (with at least
//      the two leading bytes 0x31,0x60) immediately after "reseting" the
//      FT chip and then in regular intervals. Otherwise a blue screen may
//      appear... In the code below, I make sure that every 10ms there is
//      some packet.
//
//   c) Vendor specific commands to configure the FT245 are mostly ignored.
//      Only those for reading the EEPROM are processed. See
//      usb_jtag_IN below.
//
//	1   x	_______________________	Bytes to clock out in shift mode
//			|	|	|	|	|	|
//	7	6	5	4	3	2	1	0
//	|	|	|	|	|	|	|	|
//	|	|	|	|	|	|	|	TCK
//	|	|	|	|	|	|	TMS	
//	|	|	|	|	|	CE
//	|	|	|	|	CS
//	|	|	|	TDI
//	|	|	LED	
//	|	Dir_bit = Read/Write = 1, Write = 0
//	Mode_bit = Shift = 1, BitBang = 0
//-------------------------------------------------------------------------

#define URJTAG
#include "../Gecko_library/include/AVR_Chameleon_V1.5.h"	//hardware description
#include "../Gecko_library/utility/jtag.h"		//jtag debug description
#include "hw_basic.h"
#include "jtag_map.h"
#include "eeprom.h"							//FTDI emulation
#include <avr/interrupt.h>
#include <avr/eeprom.h>						//AVR eeprom access
#include "../Gecko_library/include/usb_descriptors.h"
#include "../Gecko_library/include/usb_standard_request.h"
#include "../Gecko_library/include/usb_drv.h"	//for usb writes
#include "../Gecko_library/include/stdio_driver.h"//uart
//connect USB driver to stdio
FILE uart_str = FDEV_SETUP_STREAM(uart_std_putchar, uart_std_getchar, _FDEV_SETUP_RW);


//#define DEBUG
#ifdef DEBUG
#define debug(string,a)	printf_P(PSTR(string),a);
#else
#define debug(string,a);		//this compiles to no code for all optimizations
#endif

//-------------------------------------------------------------------------

// Global data
#define FALSE 0
#define TRUE  1
#define MaskCommandData 0x3f
#define Mode_bit	1<<7
#define Dir_bit		1<<6

unsigned char Running, WriteOnly, ClockBytes;
unsigned char RdPtr=0;
unsigned char test[64];


//-------------------------------------------------------------------------
//Start a 40ms counter. This is not an interrupt but is polled by the code 
//to detect 40ms events. When TIFR0 is set 40ms has elapsed.
void Init_40ms(void)
{
	TCCR0B = 1<<CS02 | 1<<CS00;	//divide by 1024
	TCNT0 = 256-63;				//~40ms
	TIFR0 = 0;					//clear all timeouts
								//let usb enable interrupts
}


void SPI_init(void)
{
unsigned char temp;
//Enable SPI, disable interrupt, master mode, MSB first, SPI mode 3, Fck/4
   	SPCR &= ~(1<<SPE);             			//Disable the SPI 
	SPCR |= 1<<MSTR;            			//select master prior to enabling
	SPCR = 1<<DORD | 1<<MSTR | 1<<SPR0;		//divide by 16, enable SPI master  

	SPCR &= ~(1<<CPOL);						//This works on UrJTAG in SPI mode
	SPCR &= ~(1<<CPHA);
// Clear the SPIF flag
  	temp = SPSR;         
  	temp = SPDR;   
}

#ifdef DEBUG
void print_eeprom(void)
{
unsigned int i;
	for(i=16;i<80;i++){
		debug("eeprom[%d]",i);
		debug(" = 0x%x\n",eeprom_read_byte((unsigned char*)i)); 
	}
}
#endif

//Handle bulk IN endpoint 1
//This routine is called frequently to process IN packets that contain
//the JTAG TDO data. Each IN packet is pre-pended with two bytes
//of modem status. If data is available to return (pending > 0) 
//then it is copied to the IN packet and sent to the host.
//The IN data results from the OUT packet requests but is handled here.
//
void usb_jtag_IN(void) 				//Called repeatedly while device is idle
{
unsigned char bytes_to_send;//,temp;
	if(!Running) return;			//wait for vendor request to start bulk processing
	Endpoint_select_endpoint(EP1);
	if (Usb_FIFO_available()){		//Can endpoint be written to?
		INpkts++;
		if(Pending >0){
			Usb_write_byte(0x31);	//First two bytes expected by FTDI
			Usb_write_byte(0x60);
			if(Pending > 0x3B){ 	//send less than IN ep1 size
				bytes_to_send = 0x3B; Pending -= bytes_to_send;
			}else{
				bytes_to_send = Pending; Pending = 0;
			};
			while(bytes_to_send--){
				Usb_write_byte(OutBuffer[RdPtr]);
				RdPtr = (RdPtr+1) & (OUTBUFFER_LEN-1);	//modulo wrap counter
			}
			TCNT0 = 250;				//Make a short transfer soon FTDI
		}else{
//Kick the FTDI host driver with a dummy two byte modem status packet
//every 40ms. This is an emulation feature of the FT245BM and it just
//keeps the host happy.
			if (TIFR0&(1<<TOV0)){		//
				Usb_write_byte(0x31);
				Usb_write_byte(0x60);
				TCNT0 = 256-63;			//reset counter
				TIFR = 1<<TOV0;			//reset overflow flag	
			};
		};
	};//if(Usb_FIFO_available()
	if(UEBCLX !=0){
		Usb_ack_in_ready();				//tell h/w packet is ready
	};
}

//Handle BULK OUT endpoint 2
//Called repeatedly while device is idle
//Process the host OUT packets commands and perform either a bit-bang or
//shift mode JTAG operation. This feature emulates the CPLD functions.
//This is called at interrupt level when an OUT packet is detected.
void usb_jtag_OUT(void) 					
{
unsigned char cmd, data, temp;
    Endpoint_select_endpoint(EP2);
	rx_counter = Usb_byte_counter();					//determine packet size

	debug("\nOUT>>>>>rx_counter = %d",rx_counter);
	OUTpkts++;

    if (rx_counter && ((Pending+rx_counter) < (OUTBUFFER_LEN-1))){
		for(rx_counter=rx_counter;rx_counter!=0;rx_counter--){		//process pkt
			if(ClockBytes > 0){							//Shift mode transfer
				data = Usb_read_u08();
				ClockBytes--;							//
				if(WriteOnly){ 							//Write 8 bits from cmd
					ProgIO_ShiftOut(data);
           		}else{ 									//RdWr 8 bits from cmd
					OutputByte(temp =ProgIO_ShiftInOut(data));
				}		
	     	}else{
				cmd = Usb_read_u08();					//Read command byte				
				WriteOnly = (cmd & Dir_bit) ? FALSE : TRUE;	//Rd/-Wr
				if(cmd & Mode_bit){						//Shift
					ClockBytes = cmd & MaskCommandData;	//length of shift
				}else{									//Bit Bang
					if(WriteOnly){
						ProgIO_Set_State(cmd);			//write I/O 
					}else{
						OutputByte(ProgIO_Set_Get_State(cmd));//RdWr I/O
					}
				};
			};//end transfer type shift or bit bang
		};//byte processing complete
	};//if (!rx_counter
	Usb_ack_receive_out();								//acknowledge packet
};

//-------------------------------------------------------------------------
// Vendor Requests
// These handle the control requests to endpoint 0
// OUT request:
//		Look for a bmRequest type to start the JTAG bulk processing 
//		handlers. Until this control code is received the AVR will not 
//		respond to any bulk IN/OUT requests.
// IN request:
//		Send a two byte modem status followed by the two eeprom bytes 
//		pointed to by the wIndex field. If bmRequest is not a request for 
//		eeprom data then just send the two modem status bytes.
//-------------------------------------------------------------------------
unsigned char app_vendor_cmd(void)
{
unsigned char addr,temp;
// OUT request:
	if ((bmRequestType & bmRT_DIR_MASK) == bmRT_DIR_OUT){
		if(bmRequest == GET_STATUS){
			printf_P(PSTR("UrJTAG starting..."));
			temp = UENUM;
			Endpoint_select_endpoint(EP1);
			printf_P(PSTR("EP1 0x%x"),UEINTX & (1<<TXINI | 1<<RXOUTI));
			Endpoint_select_endpoint(EP2);
			printf_P(PSTR("EP2 0x%x"),UEINTX & (1<<TXINI | 1<<RXOUTI));
			UENUM = temp;
			Pending = 0;			//reset buffers
			Running = TRUE;			//start the interupt processing
			INpkts = 0;
			OUTpkts = 0;
			JTAGclocks = 0;
		};
		return TRUE;
	}

//IN request:
	Endpoint_select_endpoint(EP0);
	if(bmRequest == 0x90){
		addr = (wIndex*2) & 0x7F;
		Usb_write_byte(eeprom_read_byte((unsigned char*)(addr+eeOffset)));
		Usb_write_byte(eeprom_read_byte((unsigned char*)(addr+eeOffset+1)));
	}else{
		Usb_write_byte(0x36);			//dummy data
		Usb_write_byte(0x83);			//why isn't this 0x31,0x60??
	}
	Usb_ack_in_ready();					//tell h/w packet is ready
	while( !Endpoint_OUT_received_int());//wait for host to ask for it
   	Usb_ack_receive_out();				//send packet
	return TRUE;
}

//-------------------------------------------------------------------------
int main(void)
{
	usb_init(16,0x09fb,0x6001);				//usb-blaster
//#ifdef DEBUG
	uart_init(115200);
	stdout = stdin = stderr = &uart_str;	//connect serial I/O stream
	printf_P(PSTR("Starting...\n"));
//#endif
	Init_40ms();
	JTAG_data = 0;
	ClockBytes = 0;
	SPI_init();
  	ProgIO_Init();
	PgmOff();
  	eeprom_init();
	Vendor_Callback(&app_vendor_cmd);
	usb_IN_Callback(&usb_jtag_IN);
	usb_OUT_Callback(&usb_jtag_OUT);
  	while(1);								//wait for interrupts
}




