// HID.c
//
// This file contains the USB control (EP0) functions corresponding
// to the HID enumeration process. The descriptor tables are contained
// in the usb_descriptors.c file.

//_____ I N C L U D E S ____________________________________________________
#include "include/HID_api.h"
#include "include/usb_standard_request.h"
#include <avr/io.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include "include/usb_api.h"
#include "include/usb_drv.h"
#include "include/stdio_driver.h"
#include "include/usb_debug.h"

// initial EEPROM values for the Keyboard
u08 key_Values[2] EEMEM;
//__________________________________________________________________________
//			HID Request Processing
//__________________________________________________________________________

//------------------------------------------------------------------------
// This function is called by the standard usb read request function when
// the Usb request is not supported. This function returns TRUE when the
// request is processed. This function returns FALSE if the request is not
// supported. In this case, a STALL handshake will be automatically
// sent by the standard usb read request function.
//
u08 usb_HID_request(u08* type, u08* request)
{	
u08  descriptor_type, string_type;
	debug_hid("usb_HID_request%c\n",' ');
	debug_hid("(u08)*type 0x%x  ",(u08)*type);	
	debug_hid("(u08)*request 0x%x\n",(u08)*request);	
  	string_type     = LSB(wValue);
	descriptor_type = MSB(wValue);
	
	switch(*request)
	{
		case GET_DESCRIPTOR:
			debug_enum("     GET_DESCRIPTOR%c",' ');
			switch (descriptor_type)
			{
			    case REPORT:
					hid_get_report();
					debug_enum("     REPORT%c",' ');
					return TRUE;
			    case hid:
			    default:
					return FALSE;
			}
		case SET_CONFIGURATION:
			debug_enum("     SET_CONFIGURATION%c",' ');
			switch (descriptor_type)
			{
			   case SET_REPORT:
					hid_set_report();
					debug_enum("     SET_REPORT%c",' ');
					return TRUE;
			   default:
					return FALSE;
			}
		case GET_INTERFACE:
			usb_hid_get_interface();
			debug_enum("     GET_INTERFACE%c",' ');
			return TRUE;
			
		case REQ_KeyValues:
			hid_host_write_word();
			return TRUE;
	
		default:
			return FALSE;
	}
}

// 
// This function manages 2 byte host data writes.
//
void hid_host_write_word(void)
{
	Endpoint_ack_setup_receive();

	while (!Endpoint_OUT_received_int()){};
	eeprom_write_byte(&key_Values[0], Usb_read_u08());
	eeprom_write_byte(&key_Values[1], Usb_read_u08());
	debug_hid("hid_host_write_word: byte 0%d\n",key_Values[0]);
	debug_hid("hid_host_write_word: byte 1%d\n",key_Values[1]);
	Usb_ack_receive_out();
	while (!(Is_usb_read_control_enabled()));
	
//Allow watchdog timer to restart uC
	asm ("WDR");							//stop watchdog timer
	MCUSR= !(1<<WDRF);						//Wdt_clear_flag
	WDTCSR |= (1<<WDCE) | (1<<WDE);		//Wdt_change_enable
	WDTCSR = (1 << WDE);					//Wdt_enabled 16ms
	Usb_send_control_in();					//ZLP for STATUS 
}


// 
// This function manages the hid get report request.
//
void hid_get_report(void)
{
	debug_hid("     hid_get_report: LSB(wIndex)=0x%x \n",LSB(wIndex));
	debug_hid("     hid_get_report: MSB(wIndex)=0x%x \n",MSB(wIndex));	
	if(LSB(wIndex)==0){
		data_to_transfer = (int)(HID_report[0].size);
		pbuffer = (u08*)HID_report[0].report_ptr;
		debug_hid("     hid_get_report: mouse report%c\n",' ');
	}else if(LSB(wIndex)==1){
		data_to_transfer = (int)(HID_report[1].size);
		pbuffer = (u08*)HID_report[1].report_ptr;
		debug_hid("     hid_get_report: keyboard report%c\n",' ');
	}else{
		data_to_transfer = (int)(HID_report[2].size);
		pbuffer = (u08*)HID_report[2].report_ptr;
		debug_hid("     hid_get_report: extra report%c\n",' ');
	}
	debug_hid("     hid_get_report: data_to_transfer=0x%x \n",data_to_transfer);
	debug_hid("     hid_get_report: pbuffer=0x%x \n",pbuffer);
	handle_get_request();
}

//
// This function manages hit set report request.
//
void hid_set_report (void)
{
	Endpoint_ack_setup_receive();
	Usb_send_control_in();
	debug_ep("hid_set_report1: ep#%d\n",UENUM);
	while(!Endpoint_OUT_received_int());
	Usb_ack_receive_out();
	debug_ep("hid_set_report2: ep#%d\n",UENUM);//ep send
	Usb_send_control_in();
}

// 
// This function manages hid set idle request.
//??????????THIS LOOKS LIKE GET INTERFACE?????????????
void usb_hid_set_idle (void)
{
	Endpoint_ack_setup_receive();
	Usb_send_control_in();				//ZLP for STATUS
	debug_ep("usb_hid_set_idle: ep#%d\n",UENUM);
	while(!Is_usb_in_ready());
}


// 
// This function manages hid get interface request.
//
void usb_hid_get_interface (void)
{
	Endpoint_ack_setup_receive();
	Usb_send_control_in();				//ZLP for STATUS 
	debug_ep("usb_hid_get_interface: ep#%d\n",UENUM);
	while(!Is_usb_in_ready());
}

//__________________________________________________________________________
//			HID API
//__________________________________________________________________________

void HID_Init(void)
{	
	usb_user_request_callback(usb_HID_request);
}
