//	usb_debug.c
//
// The following debug routines can be used to help isolate USB problems
//when developing code. Each routine is conditionally included to limit
//code size. The DEBUG flag must be set for any of these routines to be 
//available. Additionaly the INIT,EP,CDC,HID flags can also be set to
//include specific routines as required. The hierarchy is as follows:
//	DEBUG
//				debug_to_uart()
//				debug_HexDump()
//		DEBUG_INIT
//				debug_ep_init()
//		DEBUG_EP
//				debug_dev_interupt()
//				debug_ep_interupt()
//		DEBUG_CDC
//				debug_CDC_line_code()
//
//
//_____  I N C L U D E S ___________________________________________________ 
#include <avr/io.h>  
#include "include/typedefs.h"
#include "include/usb_drv.h"
#include "include/usb_descriptors.h"
#include <stdio.h>
#include "include/stdio_driver.h"
#include "include/usb_debug.h"

#ifdef DEBUG
//connect USB driver to stdio
FILE uart_str = FDEV_SETUP_STREAM(uart_std_putchar, uart_std_getchar, _FDEV_SETUP_RW);

//This routine connects the uart to the STDIO print stream and sets the baud
//rate to 115200bps
void debug_to_uart(void)
{
	uart_init(115200);
	stdout = stdin = stderr = &uart_str;		//connect serial I/O stream
	printf_P(PSTR("\nUart debug started...\n"));
}

//Print the content of size bytes of buf. in hex format
//The results are displayed as lines of 16 bytes.
void debug_HexDump(char* buf,int size)
{
int i,j;
	for(i = 0; i<=(size/16); i++){
		printf_P(PSTR("\n"));
		if(size-(i*16) >16){					//print full 16 byte lines
			for(j=0; j<16; j++){
				printf_P(PSTR(" 0x%2.2x"),buf[i*16+j]);
			}
		}else{
			for(j=0; j<(size-(i*16)); j++){		//print shorter remainder line
				printf_P(PSTR(" 0x%2.2x"),buf[i*16+j]);
			}
		}
	}
	printf_P(PSTR("\n"));
}
#else
void debug_to_uart(void){}						//In case the option is disabled
void debug_HexDump(char* buf,int size){}		//compile to nothing
#endif

#ifdef DEBUG_INIT
//It is often useful to decode the configuration of the AVR's DPRAM fifo. This
//routine prints each data field in the config0,1 registers as english text.
//Note that the configuration values are only valid after the USB device has 
//been configured, before that they are undefined. That is to say if you call
//this routine from your code make sure that the device is configured first.
void debug_ep_init(unsigned char cfg0,unsigned char cfg1)
{
//Decode cfg0	
	printf_P(PSTR("\nCFG0 Type = "));
	switch((cfg0>>6)&0x03){						//decode transfer type
		case 0x00:
			printf_P(PSTR("CONTROL     "));
			break;
		case 0x01:
			printf_P(PSTR("ISOCHRONOUS "));
			break;
		case 0x02:
			printf_P(PSTR("BULK        "));
			break;
		case 0x03:
			printf_P(PSTR("INTERRUPT   "));
			break;
		default:
			printf_P(PSTR("UNKNOWN     "));
			break;
	}	
	printf_P(PSTR("Direction = "));				//decode transfer direction
	if((cfg0 & 0x01) == 0){
		printf_P(PSTR("OUT "));
	}else{
		printf_P(PSTR("IN"));
	}
//Decode cfg1							
	printf_P(PSTR("\nCFG1 Size = "));
	switch((cfg1>>4)&0x07){						//decode ep size in bytes
		case 0x00:
			printf_P(PSTR("8"));
			break;
		case 0x01:
			printf_P(PSTR("16"));
			break;
		case 0x02:
			printf_P(PSTR("32"));
			break;
		case 0x03:
			printf_P(PSTR("64"));
			break;
		case 0x04:
			printf_P(PSTR("128"));
			break;
		case 0x05:
			printf_P(PSTR("256"));
			break;
		case 0x06:
			printf_P(PSTR("512"));
			break;	
		case 0x07:
			printf_P(PSTR("Reserved"));
			break;			
	}	
	printf_P(PSTR(" Banks = "));				//decode number of banks per ep
	if((cfg1>>2 & 0x01) == 0){
		printf_P(PSTR("ONE"));
	}else{
		printf_P(PSTR("TWO"));
	}
	printf_P(PSTR(" Allocated = "));			//decode if the ep is allocated
	if((cfg1>>1 & 0x01) == 0){
		printf_P(PSTR("NO"));
	}else{
		printf_P(PSTR("YES"));
	}	
}
#else
void debug_ep_init(unsigned char cfg0,unsigned char cfg1){}
												//In case the option is disabled
												//compile to nothing
#endif

#ifdef DEBUG_EP
//Decode device interupts and display as english text
void debug_dev_interupt(void)
{
	printf_P(PSTR("UERST 0x%x\n"),UERST);
	printf_P(PSTR("UDINT ="));
	if(UDINT&1<<UPRSMI)	{printf_P(PSTR(" Upstrean resume "));};
	if(UDINT&1<<EORSMI)	{printf_P(PSTR(" End of resume   "));};
	if(UDINT&1<<WAKEUPI){printf_P(PSTR(" Wakeup          "));};
	if(UDINT&1<<EORSTI)	{printf_P(PSTR(" End of reset    "));};
	if(UDINT&1<<SOFI)	{printf_P(PSTR(" Start of frame  "));};
	if(UDINT&1<<SUSPI)	{printf_P(PSTR(" Suspend         "));};
	printf_P(PSTR("\n"));
}

//Decode endpoint interupts and display as english text
void debug_ep_interupt(void)
{
u08 i;
u08 EP_temp;
	EP_temp = UENUM;							//SaveEP();
	for(i=0;i<5;i++){
		Endpoint_select_endpoint(i);
		if((UEINTX&(1<<RXOUTI)) | (UEINTX&(1<<TXINI)) | (UEINTX&(1<<RXSTPI)) ){
			if(UEINTX){
				printf_P(PSTR("EP%d enabled "),UENUM);
				printf_P(PSTR("time = %d "),debug_timer);
				if(UEINTX&(1<<FIFOCON))	{printf_P(PSTR(" FIFOCON    "));};
				if(UEINTX&(1<<NAKINI))	{printf_P(PSTR(" Nak IN     "));};
				if(UEINTX&(1<<RWAL))	{printf_P(PSTR(" R/W allowed"));};
				if(UEINTX&(1<<NAKOUTI))	{printf_P(PSTR(" Rx Nak out "));};
				if(UEINTX&(1<<RXSTPI))	{printf_P(PSTR(" Rx stop    "));};
				if(UEINTX&(1<<RXOUTI))	{printf_P(PSTR(" Rx OUT     "));};
				if(UEINTX&(1<<STALLEDI)){printf_P(PSTR(" Stalled    "));};
				if(UEINTX&(1<<TXINI))	{printf_P(PSTR(" Tx IN      "));};
				printf_P(PSTR("\n"));	
			}
		}
	}
	UENUM = EP_temp;							//RestoreEP();	
}
#else
void debug_dev_interupt(void){}					//In case the option is disabled
void debug_ep_interupt(void){}					//compile to nothing
#endif

#ifdef DEBUG_CDC
//Decode the USB CDC line coding and display the stop bits, party and line rate
void debug_CDC_line_code(void)
{
	printf_P(PSTR("     Baud: %lu bps "),(u32)line_coding.dwDTERate);
	switch(line_coding.bCharFormat){
	case 0:
		printf_P(PSTR(" 0"));
		break;
	case 1:
		printf_P(PSTR(" 1.5"));
		break;
	case 2:
		printf_P(PSTR(" 2"));
		break;
	};
	printf_P(PSTR(" stop bits Parity: "));
	switch(line_coding.bParityType){
	case 0:
		printf_P(PSTR("none"));
		break;
	case 1:
		printf_P(PSTR("odd"));
		break;
	case 2:
		printf_P(PSTR("even"));
		break;
	case 3:
		printf_P(PSTR("mark"));
		break;
	case 4:
		printf_P(PSTR("space"));
		break;
	};
	printf(" data bits: %d\n",line_coding.bDataBits);
}
#else
void debug_CDC_line_code(void){}						//In case the option is disabled
													//compile to nothing
#endif

	