// usb_standard_request.c
//
// This file contains the USB control (EP0) management routines corresponding
// to the standard enumeration process (refer to chapter 9 of the USB
// specification. Non-standard requests are managed with a call to
// usb)user_read_request(). This provides a user call back for CDC,HID etc. 
// The descriptor tables are contained in the usb_descriptors.c file.
//_____ I N C L U D E S ____________________________________________________
#include <avr/io.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include "include/usb_api.h"
#include "include/usb_drv.h"
#include "include/usb_standard_request.h"
#include "include/CDC_api.h"
#include "include/stdio_driver.h"
#include "include/usb_debug.h"

//_____ P R I V A T E   D E C L A R A T I O N ______________________________
static  void	usb_get_descriptor(void);
static  void	usb_set_address(void);
static  void	usb_set_configuration(void);
static  void	usb_get_configuration(void);
static  void 	usb_vendor_request(void);

//_____ D E C L A R A T I O N ______________________________________________
static 		u08 	zlp;
volatile 	u08 	Unicode = FALSE;
volatile 	u08 	AddHeader;
			u16 	wValue,wIndex,wLength;
			u08 	bmRequestType,bmRequest;
			u08 	usb_configuration_nb;
extern 		u08	usb_connected;
extern 		S_usb_device_descriptor	usb_user_device_descriptor;
			S_line_coding line_coding;

void 		*user_request_callback;	
u08 		(*ptr)();
u08 		FIFO_depth = 0;

void usb_user_request_callback(void * fct)
{
	user_request_callback = fct;
}

//
// This function reads the SETUP request sent to the default control 
// endpoint 0 and calls the appropriate function. On exiting the 
// device is ready to manage the next request.
//
void usb_process_request(void)
{
u08	 bmRequest;

   	bmRequestType 	= Usb_read_u08();					//read setup parameters
   	bmRequest     	= Usb_read_u08();
	LSB(wValue)		= Usb_read_u08();
	MSB(wValue)		= Usb_read_u08();
	LSB(wIndex)		= Usb_read_u08();
	MSB(wIndex)		= Usb_read_u08();
	LSB(wLength)	= Usb_read_u08();
	MSB(wLength)	= Usb_read_u08();
	debug_enum("\n__________________________________",' ');
	debug_enum("\nSetup request: bmRequestType=0x%x ",bmRequestType);
	debug_enum("bmRequest=0x%x ",bmRequest);
	debug_enum("wValue=0x%x ",wValue);
	debug_enum("wIndex=0x%x ",wIndex);
	debug_enum("wLength=0x%x\n",wLength);	
	switch(bmRequestType & 0x60){						//mask request type
	case StdRequest:
		switch(bmRequest){
			case GET_DESCRIPTOR:
         		if (0x80 == bmRequestType) { 
					debug_enum("Get descriptor %c",'\n');
					usb_get_descriptor(); 
					return;};

    		case GET_CONFIGURATION:
         		if (0x80 == bmRequestType) { 
					debug_enum("Get configuration %c",'\n');
					usb_get_configuration(); 				
					return;}

			case SET_ADDRESS:
				if (0x00 == bmRequestType) { 
					debug_enum("Set address %c",'\n');
					usb_set_address(); 
					return;}

			case SET_CONFIGURATION:
         		if (0x00 == bmRequestType) { 
					debug_enum("Set configuration %c",'\n'); 
					usb_set_configuration();
					return;};
			
			default:
				debug_enum("     Unknown std_request%c",' ');
				break;
		};//end of bmRequest case "End of standard requests"
//	case ClassRequest:
//		debug_enum("Class request%c",'\n');
//		break;		
	case VendorRequest:
		debug_enum("Vendor request%c",'\n');
		usb_vendor_request();
		break;
	default: 										//un-supported request 
        if(usb_user_read_request(bmRequestType, bmRequest) == FALSE){
			debug_enum("Un-supported request%c\n",' ');
				Usb_stall_and_ack_setup();
        };
	};	//end of bmRequestType case
}

// This function manages the custom vendor requests. If a vendor_callback
// function is provided control is transfered otherwise the vendor request
// triggers a reboot of the bootlader.
//
void usb_vendor_request(void)
{
u08 (*ptr)();										//define vendor callback
	Endpoint_ack_setup_receive();					//Occurs before writing reply
	if(Usb_Vendor_Callback){						//call vendor setup handler
		debug_ep("Usb_Vendor_Callback: 0x%x\n",Usb_Vendor_Callback);
		ptr = Usb_Vendor_Callback;					//if it is defined 
		(*ptr)();									//actual call  
	}else{
		debug_ep("bootloader_start!%c",'\n');
//Comment out this line so that the Ubuntu Linux doesn't confuse the device ito re-booting
//		bootloader_start();
	}
	Usb_send_control_in();							//Send packet
	debug_ep("usb_process_request: VendorRequest ep#%d\n",UENUM);
	while(!Is_usb_in_ready());						//waits for status phase done
}

// This function manages the SET ADDRESS request. When complete, the device
// will filter requests using the new address.
//
void usb_set_address(void)
{
   	Endpoint_ack_setup_receive();
	while(!Is_usb_in_ready());						//Wait for the control fifo available
   	Usb_send_control_in();              			//send a ZLP for STATUS phase
	debug_ep("usb_set_address: ep#%d\n",UENUM);
	Usb_configure_address(LSB(wValue));				//set address
   	while(!Is_usb_in_ready());						//waits for status phase done(fifo empty)
   	Usb_enable_address();  							//before using the new address 
	debug_enum("     usb_set_address: address=0x%x\n",LSB(wValue));
}

// This function manages the SET CONFIGURATION request. If the selected
// configuration is valid, this function call the usb_user_endpoint_init()
// function that will configure the endpoints following the configuration
// number.
//
void usb_set_configuration( void )
{
u08 i,EP_temp;
u08 configuration_number;
    Endpoint_ack_setup_receive();
   	configuration_number = LSB(wValue);
   	if (configuration_number <= NB_CONFIGURATION){
      		usb_configuration_nb = configuration_number;
   	}else{
		Usb_stall_and_ack_setup();
   	}
   	Usb_send_control_in();                    		//< send a ZLP for STATUS phase
	debug_ep("usb_set_configuration: ep#%d\n",UENUM);
   	usb_user_endpoint_init(usb_configuration_nb);  	//< endpoint configuration
   	Usb_set_configuration_action();
	debug_enum("     usb_set_configuration: config=0x%x\n",configuration_number);
	
	EP_temp = UENUM;								//SaveEP();
	debug_ep("Trace EP state after configuartion",' ');
	for(i=0;i<5;i++){
		Endpoint_select_endpoint(i);
		debug_ep("",' ');
	}
	UENUM = EP_temp;								//RestoreEP();		
	
	Endpoint_ack_setup_receive();					//dnc attempt to fix keypad 
}

//This function returns the user string descriptors  
//The data for the strings is stored in Flash memory as a null
//terminated string. The USB protocol requires a 16 bit non-zero
//terminated UNICODE string; so we we set a UNICODE flag and
//correct the output as it is played out in the handle_get_descriptor()
//routine.
void usb_string_descriptor(u08 string)
{ 
	Unicode = FALSE;
    switch (string){
        case LANG_ID:
          	data_to_transfer = sizeof (usb_user_language_id);
          	pbuffer = &(usb_user_language_id.bLength);
			debug_enum("     LANG_ID%c",' ');
			break;
        case MAN_INDEX:
			Unicode = TRUE;							//convert to 16 bit unicode
			data_to_transfer=string_desc[0].size;
			pbuffer = (u08*)string_desc[0].string;
			debug_enum("     MAN_ID%c",' ');
			break;
        case PROD_INDEX:
			Unicode = TRUE;							//convert to 16 bit unicode
			data_to_transfer=string_desc[1].size;
			pbuffer = (u08*)string_desc[1].string;
			debug_enum("     PROD_ID%c",' ');
			break;
        case SN_INDEX:
			Unicode = TRUE;							//convert to 16 bit unicode
			data_to_transfer=string_desc[2].size;
			pbuffer = (u08*)string_desc[2].string;
			debug_enum("     SN_ID%c",' ');
			break;
        default:
			debug_enum("     Unknown string%c\n",' ');
          	Usb_stall_and_ack_setup();
	}
}

// This function manages the GET DESCRIPTOR request. The device descriptor,
// and the configuration descriptor are supported. All
// other descriptors must be supported by the usb_user_get_descriptor
// function. Only 1 configuration is supported. All data is stored in Flash.
//
void usb_get_descriptor(void)
{
u08  descriptor_type,string_type;
//>>>>>>>>>>Is the ZLP definition required It is handled later in this code
   	zlp             = FALSE;                 		//no zero length packet 
   	string_type     = LSB(wValue); 
   	descriptor_type = MSB(wValue);
	debug_enum("     usb_get_descriptor: string_type=0x%x ",string_type);
	debug_enum("descriptor_type=0x%x\n",descriptor_type);
   	switch (descriptor_type){
    	case DEVICE_DESCRIPTOR:
      		data_to_transfer = Usb_get_dev_desc_length; //size
      		pbuffer          = Usb_get_dev_desc_pointer;//data
			debug_enum("     DEVICE_DESCRIPTOR%c",' ');
      		break;
    	case CONFIGURATION_DESCRIPTOR:
     		data_to_transfer = (int)(conf_desc[0].size); //size
      		pbuffer          = (u08*)conf_desc[0].conf_desc_ptr;//data
			debug_enum("     CONFIGURATION_DESCRIPTOR%c",' ');
      		break;
		case STRING_DESCRIPTOR:
			debug_enum("     STRING DESCRIPTOR%c",' ');
			usb_string_descriptor(string_type);
			break;
		case DEVICE_QUALIFIER:
			debug_enum("     DEVICE QUALIFIER%c",'\n');
		default:
			debug_enum("     Unknown descriptor%c",' ');
			Usb_stall_and_ack_setup();
			return;
   	}
	handle_get_request();
//This code looks to see if we have received a second setup request while 
//processing the current request. If so we nee to stall the host so that
//it resends the request after we return from this interrupt; otherwise we 
//miss the setup request.
/*	if(Endpoint_setup_received_int()){
		Endpoint_enable_stall();
		Endpoint_ack_setup_receive();
	}*/
}

// This code creates a Unicode packet in the USB fifo. The first time this 
// is called the AddHeader flag should be set to append the 2 byte Unicode
// header to the packet. That is the length and string descriptor. 
// Subsequent data consists of 16 bit Unicode in the form of the byte
// followed by a 0x00 byte.
//
void Send_UNICODE(void)
{
	FIFO_depth +=2;
	if(AddHeader){									//length=2xbytes+hdr
		Usb_write_byte(wLength*2+2);
		Usb_write_byte(STRING_DESCRIPTOR);
		AddHeader = FALSE;							//only add hdr once
		debug_enum(" 0x%x ",wLength*2+2);
	}else{
		debug_enum("%c",pgm_read_byte(pbuffer));
		Usb_write_byte(pgm_read_byte(pbuffer++));	//write low byte
		Usb_write_byte(0x00);						//write high byte
		wLength --;		
	};
}
// This code handles the ram based VID and PID as setup by the usb_init()
// function. The Flash based descriptors stored in descriptor.c are 
// dynamically over-written with the SRAM values.
void Dynamic_VID_PID(void)
{
//Handle ram based VID and PID
	if(pbuffer == (u08 *)&usb_dev_desc.idVendor){
		Usb_write_byte((u08)(Vid&0xFF));
		Usb_write_byte((u08)(Vid>>8 &0xFF));
		Usb_write_byte((u08)(Pid&0xFF));
		Usb_write_byte((u08)(Pid>>8 &0xFF));
		debug_enum("0x%x ",(u08)(Vid&0xFF));
		debug_enum("0x%x ",(u08)(Vid>>8&0xFF));
		debug_enum("0x%x ",(u08)(Pid&0xFF));
		debug_enum("0x%x ",(u08)(Pid>>8&0xFF));
		pbuffer +=4;								//send VID 2 bytes plus
		wLength -=4;								//send PID 2 byte = 4
		FIFO_depth +=4;
	}
}

void Format_hex_print(void)
{
// Format the debug string to show 8 hex characters per line
	debug_enum("0x%x ",pgm_read_byte(pbuffer));
//	debug_enum("--[0x%x] ",FIFO_depth);	
	if ((FIFO_depth&0x07) == 7){
		if(FIFO_depth != EP_CONTROL_LENGTH){
			if((wLength!=1)){
				debug_enum("\n     Data:%c",' ');
			};
		};
	};
}

void handle_get_request(void)
{
	Endpoint_ack_setup_receive();					//acknowledge the setup request
	debug_enum("\nCSetupRx--UEINTX &= ~(1<<RXSTPI);",' ');
	if (wLength > data_to_transfer){				//limit the packet size to the lesser
		wLength = data_to_transfer ;				//of datato transfer or wLength
	}
	AddHeader = TRUE;								//Add a header of 
													//(length,STRING) to  
													//the UNICODE playback
	while(wLength){									//process the entire transfer(s)
		FIFO_depth = 0;
		while(!Is_usb_read_control_enabled()){		//wait for the control fifo available	
			if(Endpoint_OUT_received_int()){
				debug_enum("\nCSOl--UEINTX &= ~(1<<RXOUTI);",' ');
				UEINTX &= ~(1<<RXOUTI);
				return;
			}
		}
		debug_ep(" UEINTX & (1 << TXINI)=%d ",(UEINTX & (1 << TXINI)));	
		debug_enum("\n     byte_counter()=0x%x ",Usb_byte_counter());	
		debug_enum("wLength=0x%x ",wLength);
		debug_enum("pbuffer=0x%x\n     Data: ",pbuffer);				
		while(wLength && (FIFO_depth<EP_CONTROL_LENGTH)){//no bigger than EP0
			if(Unicode){							//Convert to UNICODE?
				Send_UNICODE();
			}else{
				Dynamic_VID_PID();					//replace Flash values with sram VID/PID
				Usb_write_byte(pgm_read_byte(pbuffer));//write byte
				Format_hex_print();
				pbuffer++;
				wLength --;
				FIFO_depth++;
			}
		}
		zlp = (FIFO_depth == EP_CONTROL_LENGTH);
		debug_enum("\nCSI--UEINTX &= ~(1<<TXINI);",' ');
		debug_ep(" handle_get_request:SEND",' ');	
		Usb_send_control_in();						//Send packet
	}
			Unicode = FALSE;						//Only set for string descriptors

	if(zlp) { 
		debug_enum("handle_get_request:ZLP %c",'\n');
		while(!Is_usb_read_control_enabled());		//wait for the control fifo available
		Usb_send_control_in();
		debug_enum("\nZLPCSI--UEINTX &= ~(1<<TXINI);",' ');
	}
	while(!Endpoint_OUT_received_int());
	debug_enum("\nCSO--UEINTX &= ~(1<<RXOUTI);",' ');
	UEINTX &= ~(1<<RXOUTI);
	debug_ep("handle_get_request:ACK\n",' ');//ep send
}

// This function manages the GET CONFIGURATION request. The current
// configuration number is returned.
//
void usb_get_configuration(void)
{
   	Endpoint_ack_setup_receive();
   	Usb_write_byte(usb_configuration_nb);
   	Usb_ack_in_ready();
	debug_ep("usb_get_configuration1\n",' ');//ep send
   	while( !Endpoint_OUT_received_int() );
   	Usb_ack_receive_out();	
	debug_ep("usb_get_configuration2\n",' ');//ep send
	debug_enum("     usb_get_configuration: nb=0x%x\nData: ",usb_configuration_nb);
}

//__________________________________________________________________________
// This function is called by the standard usb read request function when
// the Usb request is not supported. This function returns TRUE when the
// request is processed. This function returns FALSE if the request is not
// supported. In this case, a STALL handshake will be automatically
// sent by the standard usb read request function.
//
Bool usb_user_read_request(u08 type, u08 request)
{	
	ptr = user_request_callback;
	return((*ptr)(&type,&request));
}

