// The USB task checks the arrival of new requests from the USB Host.
// Other class specific requests are also processed in this file.
// This file manages all the USB events:
// Suspend / Resume / Reset / Start Of Frame / Wake Up / Vbus events/ Id transition
//
//_____  I N C L U D E S ___________________________________________________
#include <avr/io.h>
#include <avr/interrupt.h> 
#include "include/usb_api.h"
#include "include/usb_task.h"
#include "include/usb_drv.h"
#include "include/usb_standard_request.h"


//
// This function enables the USB controller and init the USB interrupts.
// The aim is to allow the USB connection detection in order to send
// the appropriate USB event to the operating mode manager.
//
void usb_task_init(void)
{
	sei();											//enable interrupts
	Usb_force_device_mode();
	Usb_select_device();
	Endpoint_select_endpoint(EP_CONTROL);
	Usb_enable();
	Usb_enable_vbus_pad();
	Usb_enable_vbus_interrupt();	 
	UDIEN |= (1<<EORSTE);							//Enable reset interrupt
	usb_start_device();	
}

//
// This function enables the USB controller and init the USB interrupts.
// The aim is to allow the USB connection detection in order to send
// the appropriate USB event to the operating mode manager.
//
void usb_start_device (void)
{
#if defined __AVR_AT90USB1287__
   	Usb_enable_regulator();
	if(CPU == 8){
		PLLCSR = ((0<<PLLP2)|(1<<PLLP1)|(1<<PLLP0)|(1<<PLLE));	//8MHz
	}else if(CPU == 16){
		PLLCSR = ((1<<PLLP2)|(0<<PLLP1)|(1<<PLLP0)|(1<<PLLE));	//16MHz
	}
#elif defined __AVR_ATmega32U4__
  	Usb_enable_regulator();
	PLLFRQ = (1<<PDIV1 | 1<<PDIV3);								//96MHz PLL
	PLLFRQ |= 1<<PLLUSB;										//48MHz to USB
	if(CPU == 8){
		PLLCSR &= ~(1<<PINDIV);
	}else if(CPU == 16){
		PLLCSR |= 1<<PINDIV;
	}
	PLLCSR |= 1<<PLLE;

#elif defined __AVR_AT90USB162__
	if(CPU == 8){
		PLLCSR = ((0<<PLLP2)|(0<<PLLP1)|(0<<PLLP0)|(1<<PLLE));	//8MHz
	}else if(CPU == 16){
		PLLCSR = ((0<<PLLP2)|(0<<PLLP1)|(1<<PLLP0)|(1<<PLLE));	//16MHz
	}

#endif
	while (!(PLLCSR & (1<<PLOCK)));					//wait for PLL to lock
   	Usb_unfreeze_clock();
   	UDIEN |= (1<<SUSPE);							//Enable suspend interrupt
   	usb_init_device();         						//configure  EP0
//	usb_user_endpoint_init(0);						//this is a hack... to force 
													//the device to configuration 0
													//untill a set config is requested
	UDIEN = 1<<EORSME | 1<<EORSTE | 1<<WAKEUPE;//resume,reset & wakeup int
   	Usb_attach();		
}

//
// This function is the entry point of the USB management. 
// If a Setup request occurs on the Default Control Endpoint,
// the usb_process_request() function is call 
void usb_task(void)
{
    Endpoint_select_endpoint(EP_CONTROL);
    if (Endpoint_setup_received_int()){
      	usb_process_request();
    }
}

//
// This function is called each time a USB interrupt occurs.
// The following USB events are processed:
// - VBus On / Off		-only on USB128
// - Id transition		-only on USB128
// - Start Of Frame
// - Suspend
// - Wake-Up
// - Resume
// - Reset
// The file usb_api.h defines what user action takes place for each
//interrupt.
//
SIGNAL(USB_GEN_vect)
{
u08 EP_temp;
	EP_temp = UENUM;								//SaveEP();
#if defined __AVR_AT90USB1287__ | defined __AVR_ATmega32U4__
   	if (Is_usb_vbus_transition()){					//Check for VBUS transitions
      	Usb_ack_vbus_transition();
      	if (Is_usb_vbus_high()){					//usb connected
         	Usb_vbus_on_action();
			Usb_enable_reset_interrupt();
         	usb_start_device();
      	}else{
         	Usb_vbus_off_action(); 					//usb detached
     	}
   	}
#endif
#if defined __AVR_AT90USB1287__
   	if(Is_usb_id_transition()){						// Check for ID pin transitions
    	Usb_ack_id_transition();
		Usb_id_action();
   	}
#endif

   	if (UDINT & (1<<SOFI)){							//start of frame interrupt
      	Usb_ack_sof();
      	Usb_sof_action();
   	}

   	if (UDINT & (1<<SUSPI)){						//suspend interrupt
      	Usb_ack_suspend();
      	UDIEN |= (1<<WAKEUPE);						//enable wake-up interrupt
      	Usb_ack_wake_up(); 							// clear wake up to detect next event
//      Usb_freeze_clock();DNC
      	Usb_suspend_action();
   	}

   	if (Is_usb_wake_up() && !(UDIEN&= ~(1<<WAKEUPE))){//don't do 
      	Usb_unfreeze_clock();						//this if we have already
      	Usb_ack_wake_up();							//disabled wakeup int
      	(UDIEN&= ~(1<<WAKEUPE))						//Disable wake-up interrupt();
      	Usb_wake_up_action();
   	}

   	if (UDINT & (1<<EORSMI)){						//resume interrupt
      	Usb_ack_resume();
      	Usb_resume_action();
   	}

   	if (UDINT & (1<<EORSTI)){						//reset interrupt
      	Usb_ack_reset();
      	usb_init_device();
      	Usb_reset_action();
		usb_configuration_nb=0;
   	}
	UENUM = EP_temp;								//RestoreEP();
}

