//
// Led routines
//
//This file contains an ISR and timer routine to pulse and PWM an
//AVR led. These functions should work on any AVR with an 8 bit 
//hardware timer0. The calling application needs to define a 
//led structure of the size of the number of leds. This array is
//maintained by these functions. A call to InitTimer starts the 
//ISR. After that the DisplayLed routine is called to set the 
//required display pattern.
#include "../include/usb16_V1.2.h"		//USB16 hardware definition
//#include "usb_keypad.h"						//USB_keypad hardware definition
#include "../include/typedefs.h"	//USB16 hardware definition
#include <util/delay.h>
#include <avr/interrupt.h>			//for ISR
#include "led.h"	

u08 reload;
extern S_PulseLed PulseLed[];
extern u08 leds;

void DisplayOff(u08 LED)
{
	PulseLed[LED].led = 0xff;				//indicates led off
}

//Setup timer 0 for 1ms interrupts
void InitTimer(u08 CPU){
	if(CPU == 8){
		reload = 256-125;			//1ms or 8MHz/64/125
	}else{
		reload = 256-250;			//1ms or 16MHz/64/250
	}
	TCNT0 = reload;
	TCCR0B = 1<<CS01 | 1<<CS00;		//prescale = 64
	TIMSK0 = 1<<TOIE0;				//timer on, timer 0 overflow int
	sei();
};

//This ISR reads a table of led structures and adjusts the led state based 
//on the 1ms timer interrupt.
//The PWM operates at 30fps (30ms) to ensure that there is no visible pulsing. 
//The light level is adjustable from 1-31 which represents 0-100% in 3% 
//increments. Timing for the PWM
//is controlled by the duration counter which controls how long the light
//level is displayed. The timer dictates how fast the leds shift from one 
//level to the next. The direction bit indicates whether the 
//PWM is increasing or decreasing in intensity.

SIGNAL(TIMER0_OVF_vect){
u08 volatile LED;
	TCNT0 = reload;								//keep the counter running
	for(LED=0;LED<leds;LED++){					//process each led in array
		if(PulseLed[LED].on_cntr == 0){			//on_time complete
			_SFR_IO8(PulseLed[LED].port) &= ~PulseLed[LED].led;//led off
		}else{
			PulseLed[LED].on_cntr--;			//dec counter while led is on
			_SFR_IO8(PulseLed[LED].port) |= PulseLed[LED].led;	//led on
		}
		if(PulseLed[LED].period_cntr == 0){		//period complete
			PulseLed[LED].on_cntr = PulseLed[LED].on_time;//reload on_time
			PulseLed[LED].period_cntr = PulseLed[LED].period;//reload period
		}else{
			PulseLed[LED].period_cntr--;		//wait for period to be complete
		}
		if(PulseLed[LED].duration_cntr != 0xff){//PWM

//The PWM levels are held for the duration after that they are 
//adjusted to the next light level modulo the period.
			PulseLed[LED].duration_cntr--;
			if(PulseLed[LED].duration_cntr == 0){	//wait for level to timeout
				PulseLed[LED].duration_cntr = PulseLed[LED].duration;
				if(PulseLed[LED].direction){
					PulseLed[LED].on_time ++;	//increase light level
					if(PulseLed[LED].on_time == PulseLed[LED].period){
						PulseLed[LED].on_time = 0;
					}
				}else{
					PulseLed[LED].on_time --;	//decrease light level
					if(PulseLed[LED].on_time == 0){
						PulseLed[LED].on_time = PulseLed[LED].period;
					}
				}
			}
		}
	}
};


//The range of on_time and period are 0-255ms. 
//LED - is the board name of the LED form 1-N. This defines which array element
//  	is programmed.
//on_time-is the duration of the led on time in ms. If this equals or exceeds
//		the period the led is always on.
//period-is the total on/off time for the led. The off time is 
//		defined as period-on_time. The duty cycle is on_time/period.
//port- is the physical hardware port that the led is on. PORT 
//		is defined as an ascii letter in the call function
//led - is the physical hardware bit that the led is on
//duration- is the length of time that a particular light level is held during PWM.
//		A duration of 0xff means don't PWM just pulse leds.
//direction- is up or down for increasing or decreasing light levels
void DisplayLED(u08 LED,u08 on_time, u08 period, u08 port,u08 led, u08 dur, u08 dir)
{
u08 volatile PORT;
switch(port){
	case 'B':
		PORT = 0x05;						//SFR address
		break;
	case 'C':
		PORT = 0x08;
		break;
	case 'D':
		PORT = 0x0B;
		break;
	default:
		PORT = 0x05;						//PORT B
		break;
}
	_SFR_IO8(PORT-1) |= 1<<led;				//set led bit as output
											//DDR is one address below PORT
	_SFR_IO8(PORT) |= 1<<led;				//set led on
	PulseLed[LED].port = PORT;				//Setup structure for led ISR
	PulseLed[LED].led = 1<<led;				//set led bit
	PulseLed[LED].on_cntr = on_time;		//load on timeout
	PulseLed[LED].on_time = on_time;		//load on timeout
	PulseLed[LED].period_cntr = period;		//load period timeout
	PulseLed[LED].period = period;			//load period timeout
	PulseLed[LED].duration_cntr = dur;		//load PWM duration
	PulseLed[LED].duration = dur;			//load PWM duration
	PulseLed[LED].direction = dir;			//load PWM duration
}

//Test the scan led as a capacitive photosensor. The code charges
//the sensor led and then times how long it takes to photo-discharge 
//the led. The advantage of this is that a single I/O bit can be 
//used to perform the scan function which frees up the ADC port. 
u16 SenseLight(u16 max_level)
{
u16 light_level;
u08 save_DDR, save_PORT;
	save_DDR = PGMDDR;
	save_PORT = PGMPort;
	light_level = 0;						//reset light counter
	SensorPORT |= sensor;					//pullup sensor 
	SensorDDR  |= sensor;					//charge led capacitor
	SensorDDR  &= ~(sensor);				//stop charging led
	SensorPORT &= ~(sensor);				//remove pullup on led
	while(SensorPIN&(sensor)){				//wait for photo-discharge
		light_level++;						//count photo delay
		_delay_ms(1);
		if(light_level == max_level){break;}
	}
	return light_level;						//return result
	PGMDDR = save_DDR;
	PGMPort = save_PORT;
}

//To detect key operations in all ambient light conditions
//from darkness to light requires two detection methods.
//The procedure is as follows:
// 1) Wait for darkness which indicates that either the 
//	  key is covered or the room is dark.
// 2) Turn on a backlight led to illuminate a hand touching 
//	  the switch.
// 3) Measure the reflected light. If light is reflected
//	  an object is touching the switch; otherwise the room
// 	  is just dark.
// 4) Turn off the backlight
//
//Note that if the backlight is on too long it draws power 
//     and is potentially vissible. It may require an IR 
//	   led is used.
u08 KeyPress(void)
{
volatile u16 light_level;
/*	light_level=SenseLight(max_timeout);
	while(light_level > Off_level){		//wait for darkness
		light_level=SenseLight(max_timeout);
	}
	PwrOn();*/								//Backlight on
	light_level=SenseLight(max_timeout);	//look for reflection
	if(light_level < On_level){
		PgmOff();							//Backlight off		
		return off;
	}else{
		PgmOff();							//Backlight off	
		return on;
	};

}







