-----------------------------------------------------------------------------
-- Company: Digital Lapis
-- Engineer: Doug Commons
-- 
-- Create Date:   12:49:17 01/21/2009 
-- Design Name: 	ChameleonV1.5Test
-- Target Devices: XC9572XL
-- Tool versions: ISE 11.4
-- Description: 
-- Note: This code needs the AVR CLKO fuse bit enabled for the clocking 
-- features to work. This code is designed to test the features of the
-- Chameleon board when used in conjunction with the AVR test code.
-- Functions:
--		Map the Q1 and Q2 FET enables to PB0 and PB1
--		Pulse the PGM clcok at ~4Hz 
--		Map PB0..4 to the B port connector as outputs
-- 	Provide a signature byte of 0x84
--		Provide a 16kHz and 16MHz test clock on the B port connector
--
-- When the interface is in the programming state with EN = 1 the CPLD 
-- signature byte is reset. When EN=0 then the 8 bit signature can be 
-- clocked out and read by the AVR. To ensure sync. the signature should be 
-- reset just before reading it (pulse EN high).
--
-----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ChameleonV1_5Test is
	Port (  
		PWR1,GND1 : out std_logic;
		SCA,SCL : out std_logic;
		B13,B12 : out  std_logic;
		B4,B5,B6,B7,B8,B9,B10,B11 : out std_logic;
	   RSTA : out std_logic;
		PB0,PB1,PB2,PB3,PB4 : in std_logic;
		EN : in std_logic;
		SO : out std_logic;
		SIGCK,CLK : in std_logic);
end ChameleonV1_5Test;

architecture Behavioral of ChameleonV1_5Test is
signal Signature : std_logic_vector (7 downto 0);
signal ShiftOut,ck16khz,LED : std_logic := '0';
signal counter : std_logic_vector(11 downto 0);

-----------------------------------------------------------------------------	
--	AVR Connection
--		SCA - LED cathode
--		SCL - led anode
-- 	SO - PB6
--		SIGCLK - PB5
--		EN - PB7
--		CLK - PC7 when CLKO fuse bit is set on the AVR
--		RSTA - Connected to the AVR RST- input
--
--	B Connector 
--	PB0	PWR1	1	2	
--	         	3	4	PB0
--			PB1	5	6	PB2
--			PB3	7	8	PB4
--		 SIGCK	9	10	'1'
--	       EN	11	12	16MHz
--		 16khz	13	14	GND1	PB1
	
begin
-----------------------------------------------------------------------------

	PWR1 <= PB0;	--Q1 FET
	GND1 <= PB1;	--Q2 FET
	SCA  <= LED;	--PGM LED ANODE
	SCL  <= '0';	--PGM LED CATHODE
	RSTA <= '1';	--Never reset the AVR

	B4   <= PB0;
	B5   <= PB1;
	B6   <= PB2;
	B7   <= PB3;
	B8   <= PB4;
	B9   <= SIGCK;		
	B10  <= '1';
	B11  <= EN;
	B12  <= CLK;
	B13  <= ck16khz;

-----------------------------------------------------------------------------
-- Shift out an 8 bit signature byte MSB first. This means that the 
-- AVR receives the signature byte in the correct bit order.
--
	SO <= ShiftOut;
	process(SIGCK,EN)
	begin
		if EN = '1' then		--ASYNC reset of signature
			Signature <= x"84";	
		elsif rising_edge(SIGCK) then
			ShiftOut <= Signature(7);
			Signature <= Signature(6 downto 0) & '1';
		end if;
	end process;	
-----------------------------------------------------------------------------
-- Create 16kHz clock from AVR clock out 16MHz
--
	process(CLK,EN,counter,ck16khz)
	begin	
		if EN = '1' then
			counter <= x"000";
			ck16khz  <= '0';
			LED <= '1';
		elsif rising_edge(CLK) then	
			counter <= counter + 1;
			if (counter >= 4000) then
				ck16khz <= not(ck16khz);
				LED <= not(LED);
				counter <= x"000";
			end if;
		end if;
	end process;	
	
end Behavioral;	