-----------------------------------------------------------------------------
-- Company: Digital Lapis
-- CPLD_RSTgineer: Doug Commons
-- 
-- Create Date:   12:49:17 01/21/2009 
-- Design Name: 	MKIIJTAGV1_5
-- Target Devices: XC9572XL
-- Tool versions: ISE 11.4
-- Description: 
-- Maps the MKII ICE JTAG pins to the AVR processor on the ChameleonV1.4.
--
-- When the interface is in the programming state with EN = 1 the CPLD 
-- signature byte is reset. When EN =0 then the 8 bit signature can be 
-- clocked out and read by the AVR. To ensure sync. the signature should be 
-- reset just before reading it (pulse EN high).
--
-----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity MKIIJTAGV1_5 is
	Port (  
			VCCB4,GNDB2,GNDB10 : out  STD_LOGIC;
			RSTA : OUT  STD_LOGIC;
			Tx : out  std_logic;
			Rx : in  std_logic;
			VCCEN,GNDEN : out  std_logic;
			TCK,TMS,TDI,PF6,B11 : in  STD_LOGIC;
			B8 : in STD_LOGIC;
			TDO,PF4,PF5,PF7,B12,B13,B14 : out  STD_LOGIC;
			EN : in STD_LOGIC;
			SO : out  STD_LOGIC;
			SIGCK: in  STD_LOGIC);
end MKIIJTAGV1_5;

architecture Behavioral of MKIIJTAGV1_5 is
signal Signature : STD_LOGIC_VECTOR (7 downto 0);
signal ShiftOut : STD_LOGIC;

-----------------------------------------------------------------------------	
--
--	MKII ICE Connector as an JTAG connection
--	PF4	TCK	1	2	GNDB2
--	PF6	TDO	3	4	VCCB4
--	PF5	TMS	5	6	
--					7	8	trst
--	PF7	TDI	9	10	GNDB10
--	Tx	   (rx)	11	12	'0'
--	Rx	   (tx)	13	14	'0'
	
begin
-----------------------------------------------------------------------------
-- Map ICE to AVR
	VCCEN <= '0';	--turn off fet Q1
	GNDEN <= '0';	--turn off fet Q2

	VCCB4 <= '1';	--virtual VCC
	GNDB2 <= '0';	--virtual ground
	PF4  <= TCK;	--map JTAG from J1 connector to AVR
	PF5  <= TMS;
	PF7  <= TDI;
	TDO  <= PF6;
	GNDB10 <= '0';	--virtual ground
	RSTA <= B8;		--reset to AVR 
	Tx   <= B11;	--map uart to AVR
	B13  <= Rx;		--map uart to AVR
	B12  <= '0';	--virtual ground
	B14  <= '0';	--virtual ground

-----------------------------------------------------------------------------
-- Shift out an 8 bit signature byte MSB first. This means that the 
-- AVR receives the signature byte in the correct bit order.
--
	SO <= ShiftOut;
	process(SIGCK,EN)
	begin
		if EN = '1' then		--ASYNC reset of signature
			Signature <= x"80";	
		elsif rising_edge(SIGCK) then
			ShiftOut <= Signature(7);
			Signature <= Signature(6 downto 0) & '1';
		end if;
	end process;	

end Behavioral;	